from flask import Flask, session, g, render_template_string, Response
from loguru import logger
import json
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from matplotlib.ticker import ScalarFormatter
import pandas as pd
import psycopg
import sqlalchemy
import time
import io

from app import app
from app import oidc




@app.route('/ntp/stratum-rootdisp.png')
def stratum_rootdisp_png():
    dbh = psycopg.connect()
    engine = sqlalchemy.create_engine("postgresql+psycopg://", creator=lambda: dbh)
    query = """
      select   time_bucket('5 minutes', time) as bucket,
        attributes->>'Label' as device,
        avg(cast(values->'rootdisp'->>'value'  as float)) as rootdisp,
        max(cast(values->'stratum'->>'value'  as int)) as stratum
      from measurements
      where time >= date_trunc('day', now()) AND time < date_trunc('day', now()) + '1 day'::interval and
        application = 'SNMP' and attributes->>'Label' IN ('harrison', 'david')
      group by bucket, attributes->>'Label'
      order by bucket, attributes->>'Label'
    """
    df = pd.read_sql(query, con=engine)
    
    df['rootdisp'] = df['rootdisp'] / 1e6

    
    # Extract date for title
    plot_date = df['bucket'].dt.date.iloc[0] if not df.empty else "Unknown Date"
    
    # Create figure with two side-by-side subplots
    fig, axes = plt.subplots(1, 2, figsize=(15, 5), sharex=True)
    
    for i, device in enumerate(['harrison', 'david']):
        ax1 = axes[i]
        ax2 = ax1.twinx()
        
        device_df = df[df['device'] == device]
        
        ax1.plot(device_df['bucket'], device_df['rootdisp'], 'r-', label='Root Dispersion')
        ax1.set_xlabel('Time')
        ax1.set_ylabel('Root Dispersion (ms)', color='r')
        ax1.tick_params(axis='y', labelcolor='r')
        
        ax2.plot(device_df['bucket'], device_df['stratum'], 'b-', label='Stratum')
        ax2.set_ylabel('Stratum', color='b')
        ax2.tick_params(axis='y', labelcolor='b')
        ax2.set_yticks(range(int(device_df['stratum'].min()), int(device_df['stratum'].max()) + 1))
        
        ax1.set_title(f'{device.capitalize()}')
        
        ax1.xaxis.set_major_formatter(mdates.DateFormatter('%H:%M'))
        fig.autofmt_xdate(rotation=45)
    
    fig.suptitle(f'Stratum and Root Dispersion - {plot_date}')
    fig.tight_layout()
    
    img_io = io.BytesIO()
    plt.savefig(img_io, format='png')
    img_io.seek(0)
    plt.close(fig)

    return Response(img_io, mimetype='image/png')


@app.route('/ntp/packets-load.png')
def packets_load_png():
    dbh = psycopg.connect()
    engine = sqlalchemy.create_engine("postgresql+psycopg://", creator=lambda: dbh)
    query = """
      select   time_bucket('5 minutes', time) as bucket,
        attributes->>'Label' as device,
        avg(cast(values->'load1'->>'value'  as float)) as load,
        avg(cast(values->'processed-pkts'->>'value'  as int)) as packets
      from measurements
      where time >= date_trunc('day', now()) AND time < date_trunc('day', now()) + '1 day'::interval and
        application = 'SNMP' and attributes->>'Label' IN ('harrison', 'david')
      group by bucket, attributes->>'Label'
      order by bucket, attributes->>'Label'
    """
    df = pd.read_sql(query, con=engine)
    
    
    # Extract date for title
    plot_date = df['bucket'].dt.date.iloc[0] if not df.empty else "Unknown Date"
    
    # Create figure with two side-by-side subplots
    fig, axes = plt.subplots(1, 2, figsize=(15, 5), sharex=True)
    
    for i, device in enumerate(['harrison', 'david']):
        ax1 = axes[i]
        ax2 = ax1.twinx()
        
        device_df = df[df['device'] == device]
        
        ax1.plot(device_df['bucket'], device_df['load'], 'r-', label='CPU Load')
        ax1.set_xlabel('Time')
        ax1.set_ylabel('Load', color='r')
        ax1.tick_params(axis='y', labelcolor='r')
        
        ax2.plot(device_df['bucket'], device_df['packets'], 'b-', label='Processed Packets')
        ax2.set_ylabel('Packets', color='b')
        ax2.tick_params(axis='y', labelcolor='b')
        
        ax1.set_title(f'{device.capitalize()}')
        
        ax1.xaxis.set_major_formatter(mdates.DateFormatter('%H:%M'))
        fig.autofmt_xdate(rotation=45)
    
    fig.suptitle(f'CPU Load and Processed Packets - {plot_date}')
    fig.tight_layout()
    
    img_io = io.BytesIO()
    plt.savefig(img_io, format='png')
    img_io.seek(0)
    plt.close(fig)

    return Response(img_io, mimetype='image/png')
